==> 0.3.19.tar.gz: android/src/main/java/com/samelement/iot_notification/appwidget/SimpleTextWidgetConfigureActivity.java <==
package com.samelement.iot_notification.appwidget;

import android.appwidget.AppWidgetManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RadioButton;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.preference.PreferenceManager;
import androidx.work.Data;
import androidx.work.ExistingWorkPolicy;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;

import com.google.gson.Gson;
import com.samelement.iot_notification.R;
import com.samelement.iot_notification.analytic.Analytic;
import com.samelement.iot_notification.analytic.AnalyticResult;
import com.samelement.iot_notification.appwidget.db.WidgetReaderDbHelper;
import com.samelement.iot_notification.appwidget.helper.AppWidgetHelper;
import com.samelement.iot_notification.device.Device;
import com.samelement.iot_notification.device.DeviceResult;
import com.samelement.iot_notification.device.NodeProperty;
import com.samelement.iot_notification.resource.AnalyticResource;
import com.samelement.iot_notification.resource.AnalyticsResourcesResult;
import com.samelement.iot_notification.resource.Resource;
import com.samelement.iot_notification.screen.IntentKey;
import com.samelement.iot_notification.service.worker.GetAnalyticsResourcesWorker;
import com.samelement.iot_notification.service.worker.GetAnalyticsWorker;
import com.samelement.iot_notification.service.worker.GetDevicesWorker;
import com.samelement.iot_notification.util.EncryptUtil;
import com.samelement.iot_notification.util.FileUtil;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * The configuration screen for the {@link SimpleTextWidget SimpleTextWidget} AppWidget.
 */
public class SimpleTextWidgetConfigureActivity extends AppCompatActivity {

    private int appWidgetId = AppWidgetManager.INVALID_APPWIDGET_ID;

    private Gson gson = new Gson();
    private final List<Device> devices = new ArrayList<>();
    private final List<Analytic> analytics = new ArrayList<>();
    private final List<AnalyticResource> analyticResources = new ArrayList<>();

    private Handler handlerThread;

    private DashboardsAdapter dashboardsAdapter;

    private WidgetReaderDbHelper dbHelper;
    private SharedPreferences prefs;

    private ProgressBar progressBar;
    private ListView listView;
    private LinearLayout layout;
    private LinearLayout widgetLayout;
    private TextView previewTitle;

    private int alpha = 255;
    private String color = "dark";
    private Analytic selectedAnalytic;

    public SimpleTextWidgetConfigureActivity() {
        super();
    }

    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);

        this.setTitle("Monitor Widget");

        dbHelper = WidgetReaderDbHelper.getInstance(this);
        prefs = PreferenceManager.getDefaultSharedPreferences(this);

        // Set the result to CANCELED.  This will cause the widget host to cancel
        // out of the widget placement if the user presses the back button.
        setResult(RESULT_CANCELED);

        setContentView(R.layout.simple_text_widget_configure);
        listView = findViewById(R.id.listview);
        dashboardsAdapter = new DashboardsAdapter(this, android.R.layout.simple_list_item_1, analytics);
        listView.setAdapter(dashboardsAdapter);
        listView.setOnItemClickListener(onItemClicked);

        progressBar = findViewById(R.id.progressbar);
        layout = findViewById(R.id.layout);

        widgetLayout = findViewById(R.id.widgetLayout);
        previewTitle = findViewById(R.id.appwidget_title);

        Button backButton = findViewById(R.id.backButton);
        backButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                setConfigurationVisibility(false);
            }
        });
        Button doneButton = findViewById(R.id.doneButton);
        doneButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (selectedAnalytic != null) {
                    addWidget(selectedAnalytic);
                }
            }
        });

        final TextView transparency = findViewById(R.id.transparency);

        SeekBar seekbar = findViewById(R.id.seekbar);
        seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                alpha = progress;
                int trans = (progress * 100) / 255;
                transparency.setText(trans + "%");

                widgetLayout.getBackground().setAlpha(progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        // Find the widget id from the intent.
        Intent intent = getIntent();
        Bundle extras = intent.getExtras();
        if (extras != null) {
            appWidgetId = extras.getInt(
                    AppWidgetManager.EXTRA_APPWIDGET_ID, AppWidgetManager.INVALID_APPWIDGET_ID);
        }

        // If this activity was started with an intent without an app widget ID, finish with an error.
        if (appWidgetId == AppWidgetManager.INVALID_APPWIDGET_ID) {
            finish();
            return;
        }

        // register local broadcast receiver to receive worker data
        IntentFilter filter = new IntentFilter();
        filter.addAction(IntentKey.APPWIDGET_WORKER_COMPLETED);
        LocalBroadcastManager.getInstance(this).registerReceiver(receiver, filter);

        // initiate handler
        handlerThread = new Handler() {
            @Override
            public void handleMessage(Message msg) {
                handleParsedCompleted(msg);
                super.handleMessage(msg);
            }
        };

        requestToServer();
    }

    private void addWidget(Analytic analytic) {
        String widgetTitle = analytic.getTitle();
        List<String> resourcesName = new ArrayList<>();
        List<String> topics = new ArrayList<>();
        List<String> devicesName = new ArrayList<>();
        List<String> units = new ArrayList<>();
        List<String> values = new ArrayList<>();

        resourcesName.add("No resource");
        values.add("-");
        units.add("");
        devicesName.add("");
        topics.add("");

        for (AnalyticResource analyticResource : analyticResources) {
            if (analyticResource.getAnalyticId().equals(analytic.getId())) {
                if (analyticResource.getResources().size() > 0) {
                    resourcesName.clear();
                    values.clear();
                    units.clear();
                    devicesName.clear();
                    topics.clear();

                    for (Resource res : analyticResource.getResources()) {
                        // sensor/humid
                        resourcesName.add(res.getName());
                        values.add("-");

                        Device device = res.getDevice();
                        if (device != null) {
                            devicesName.add(device.getName() + "(" + device.getSn() + ")");
                            topics.add(device.getDeveloperId() + "/" + device.getSn() + "/" + res.getParameter());
                            NodeProperty prop = AppWidgetHelper.getNodePropertyByDeviceParameter(res.getParameter(), device);
                            String unit = "";
                            if (prop != null) {
                                unit = prop.getUnit();
                            }
                            units.add(unit);
                        }
                    }
                }

                break;
            }
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date().getTime());

        for (int i = 0; i < resourcesName.size(); i++) {
            dbHelper.insert(appWidgetId + "", topics.get(i), widgetTitle, resourcesName.get(i),
                    devicesName.get(i), date, units.get(i), values.get(i),
                    color, alpha, false, AppWidgetType.WGT_MONITOR);
        }

        // It is the responsibility of the configuration activity to update the app widget
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this);
        SimpleTextWidget.updateAppWidget(this, appWidgetManager, appWidgetId);

        // Make sure we pass back the original appWidgetId
        Intent resultValue = new Intent();
        resultValue.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, appWidgetId);
        setResult(RESULT_OK, resultValue);
        finish();
    }

    private void setConfigurationVisibility(boolean isVisible) {
        LinearLayout previewLayout = findViewById(R.id.previewLayout);
        RelativeLayout analyticsLayout = findViewById(R.id.analytics_layout);
        if (isVisible) {
            previewLayout.setVisibility(View.VISIBLE);
            analyticsLayout.setVisibility(View.GONE);
        } else {
            previewLayout.setVisibility(View.GONE);
            analyticsLayout.setVisibility(View.VISIBLE);
        }
    }

    private AdapterView.OnItemClickListener onItemClicked = new AdapterView.OnItemClickListener() {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            selectedAnalytic = analytics.get(position);
            previewTitle.setText(selectedAnalytic.getTitle());
            setConfigurationVisibility(true);
        }
    };

    public void onRadioButtonClicked(View view) {
        boolean checked = ((RadioButton) view).isChecked();
        // Check which radio button was clicked
        int id = view.getId();
        if (id == R.id.radioButtonDark) {
            if (checked) {
                color = "dark";
                widgetLayout.setBackgroundColor(ContextCompat.getColor(this, R.color.colorContentDark));
            }
        } else if (id == R.id.radioButtonLight) {
            if (checked) {
                color = "light";
                widgetLayout.setBackgroundColor(ContextCompat.getColor(this, R.color.colorContentLight));
            }
        }

        widgetLayout.getBackground().setAlpha(alpha);
    }

    private void handleParsedCompleted(Message msg) {
        if (msg.what == IntentKey.PARSED_COMPLETED) {
            layout.setVisibility(View.VISIBLE);
            progressBar.setVisibility(View.GONE);
            dashboardsAdapter.notifyDataSetChanged();
        }
    }

    private void requestToServer() {
        final OneTimeWorkRequest getDevicesRequest = new OneTimeWorkRequest.Builder(GetDevicesWorker.class)
                .build();
        final OneTimeWorkRequest getAnalyticsRequest = new OneTimeWorkRequest.Builder(GetAnalyticsWorker.class)
                .build();

        final Data data = new Data.Builder()
                .putString(IntentKey.KEY_BROADCAST, IntentKey.APPWIDGET_WORKER_COMPLETED)
                .build();
        final OneTimeWorkRequest getAnalyticsResourcesRequest = new OneTimeWorkRequest.Builder(GetAnalyticsResourcesWorker.class)
                .setInputData(data)
                .build();

        WorkManager.getInstance(this)
                .beginUniqueWork("requestDataForAppWidget", ExistingWorkPolicy.REPLACE, getDevicesRequest)
                .then(getAnalyticsRequest)
                .then(getAnalyticsResourcesRequest)
                .enqueue();
    }

    private BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equals(IntentKey.APPWIDGET_WORKER_COMPLETED)) {
                String deviceUuid = intent.getExtras().getString(GetDevicesWorker.KEY);
                String analyticUuid = intent.getExtras().getString(GetAnalyticsWorker.KEY);
                String resourceUuid = intent.getExtras().getString(GetAnalyticsResourcesWorker.KEY);

                parseJsonResult(deviceUuid, analyticUuid, resourceUuid);
            }
        }
    };

    private void parseJsonResult(final String deviceUuid, final String analyticUuid, final String resourceUuid) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                // parse devices
                File path = getApplicationContext().getFilesDir();
                String devicesJson = FileUtil.readFromFile(new File(path, deviceUuid + ".json"));

                DeviceResult deviceResult = gson.fromJson(devicesJson, DeviceResult.class);

                if (deviceResult != null && deviceResult.getBody() != null) {
                    devices.clear();
                    devices.addAll(deviceResult.getBody());
                }


                // parse analytic
                String analyticsJson = FileUtil.readFromFile(new File(path, analyticUuid + ".json"));
                AnalyticResult analyticResult = gson.fromJson(analyticsJson, AnalyticResult.class);
                if (analyticResult != null && analyticResult.getBody() != null) {
                    analytics.clear();
                    analytics.addAll(analyticResult.getBody());
                    Collections.sort(analytics);
                }

                // parse resources
                String resourcesJson = FileUtil.readFromFile(new File(path, resourceUuid + ".json"));
                AnalyticsResourcesResult analyticsResourcesResult = gson.fromJson(resourcesJson, AnalyticsResourcesResult.class);
                if (analyticsResourcesResult != null && analyticsResourcesResult.getBody() != null) {
                    analyticResources.clear();
                    analyticResources.addAll(analyticsResourcesResult.getBody());
                }

                // set resource device
                for (AnalyticResource ares : analyticResources) {
                    if (ares.getResources() != null && ares.getResources().size() > 0) {
                        for (Resource res : ares.getResources()) {
                            for (Device dev : devices) {
                                if (res.getDeviceId().equals(dev.getId())) {
                                    res.setDevice(dev);
                                    break;
                                }
                            }

                        }
                    }
                }


                // delete file
                File dir = getFilesDir();
                File fileDevices = new File(dir, deviceUuid + ".json");
                fileDevices.delete();

                File fileAnalytics = new File(dir, analyticUuid + ".json");
                fileAnalytics.delete();

                File fileResources = new File(dir, resourceUuid + ".json");
                fileResources.delete();


                Message msg = new Message();
                msg.what = IntentKey.PARSED_COMPLETED;
                handlerThread.sendMessage(msg);

            }
        }).start();
    }

    @Override
    protected void onDestroy() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(receiver);
        dbHelper.close();
        super.onDestroy();
    }
}

==> 0.5.1.tar.gz: android/src/main/java/com/samelement/iot_notification/appwidget/SimpleTextWidgetConfigureActivity.java <==
package com.samelement.iot_notification.appwidget;

import android.appwidget.AppWidgetManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RadioButton;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.work.Data;
import androidx.work.ExistingWorkPolicy;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;

import com.google.gson.Gson;
import com.samelement.iot_notification.R;
import com.samelement.iot_notification.analytic.Analytic;
import com.samelement.iot_notification.analytic.AnalyticResult;
import com.samelement.iot_notification.appwidget.db.WidgetReaderDbHelper;
import com.samelement.iot_notification.appwidget.helper.AppWidgetHelper;
import com.samelement.iot_notification.device.Device;
import com.samelement.iot_notification.device.DeviceResult;
import com.samelement.iot_notification.device.NodeProperty;
import com.samelement.iot_notification.resource.AnalyticResource;
import com.samelement.iot_notification.resource.AnalyticsResourcesResult;
import com.samelement.iot_notification.resource.Resource;
import com.samelement.iot_notification.screen.IntentKey;
import com.samelement.iot_notification.service.worker.GetAnalyticsResourcesWorker;
import com.samelement.iot_notification.service.worker.GetAnalyticsWorker;
import com.samelement.iot_notification.service.worker.GetDevicesWorker;
import com.samelement.iot_notification.util.FileUtil;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * The configuration screen for the {@link SimpleTextWidget SimpleTextWidget} AppWidget.
 */
public class SimpleTextWidgetConfigureActivity extends AppCompatActivity {

    private int appWidgetId = AppWidgetManager.INVALID_APPWIDGET_ID;

    private final Gson gson = new Gson();
    private final List<Device> devices = new ArrayList<>();
    private final List<Analytic> analytics = new ArrayList<>();
    private final List<AnalyticResource> analyticResources = new ArrayList<>();

    private Handler handlerThread;

    private DashboardsAdapter dashboardsAdapter;

    private WidgetReaderDbHelper dbHelper;

    private ProgressBar progressBar;
    private LinearLayout layout;
    private LinearLayout widgetLayout;
    private TextView previewTitle;

    private int alpha = 255;
    private String color = "dark";
    private Analytic selectedAnalytic;

    public SimpleTextWidgetConfigureActivity() {
        super();
    }

    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);

        this.setTitle("Monitor Widget");

        dbHelper = WidgetReaderDbHelper.getInstance(this);

        // Set the result to CANCELED.  This will cause the widget host to cancel
        // out of the widget placement if the user presses the back button.
        setResult(RESULT_CANCELED);

        setContentView(R.layout.simple_text_widget_configure);
        ListView listView = findViewById(R.id.listview);
        dashboardsAdapter = new DashboardsAdapter(this, android.R.layout.simple_list_item_1, analytics);
        listView.setAdapter(dashboardsAdapter);
        listView.setOnItemClickListener(onItemClicked);

        progressBar = findViewById(R.id.progressbar);
        layout = findViewById(R.id.layout);

        widgetLayout = findViewById(R.id.widgetLayout);
        previewTitle = findViewById(R.id.appwidget_title);

        Button backButton = findViewById(R.id.backButton);
        backButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                setConfigurationVisibility(false);
            }
        });
        Button doneButton = findViewById(R.id.doneButton);
        doneButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (selectedAnalytic != null) {
                    addWidget(selectedAnalytic);
                }
            }
        });

        final TextView transparency = findViewById(R.id.transparency);

        SeekBar seekbar = findViewById(R.id.seekbar);
        seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                alpha = progress;
                int trans = (progress * 100) / 255;
                transparency.setText(trans + "%");

                widgetLayout.getBackground().setAlpha(progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        // Find the widget id from the intent.
        Intent intent = getIntent();
        Bundle extras = intent.getExtras();
        if (extras != null) {
            appWidgetId = extras.getInt(
                    AppWidgetManager.EXTRA_APPWIDGET_ID, AppWidgetManager.INVALID_APPWIDGET_ID);
        }

        // If this activity was started with an intent without an app widget ID, finish with an error.
        if (appWidgetId == AppWidgetManager.INVALID_APPWIDGET_ID) {
            finish();
            return;
        }

        // register local broadcast receiver to receive worker data
        IntentFilter filter = new IntentFilter();
        filter.addAction(IntentKey.APPWIDGET_WORKER_COMPLETED);
        LocalBroadcastManager.getInstance(this).registerReceiver(receiver, filter);

        // initiate handler
        handlerThread = new Handler(Looper.getMainLooper()) {
            @Override
            public void handleMessage(Message msg) {
                handleParsedCompleted(msg);
                super.handleMessage(msg);
            }
        };

        requestToServer();
    }

    private void addWidget(Analytic analytic) {
        String widgetTitle = analytic.getTitle();
        List<String> resourcesName = new ArrayList<>();
        List<String> topics = new ArrayList<>();
        List<String> devicesName = new ArrayList<>();
        List<String> units = new ArrayList<>();
        List<String> values = new ArrayList<>();

        resourcesName.add("No resource");
        values.add("-");
        units.add("");
        devicesName.add("");
        topics.add("");

        for (AnalyticResource analyticResource : analyticResources) {
            if (analyticResource.getAnalyticId().equals(analytic.getId())) {
                if (analyticResource.getResources().size() > 0) {
                    resourcesName.clear();
                    values.clear();
                    units.clear();
                    devicesName.clear();
                    topics.clear();

                    for (Resource res : analyticResource.getResources()) {
                        // sensor/humid
                        resourcesName.add(res.getName());
                        values.add("-");

                        Device device = res.getDevice();
                        if (device != null) {
                            devicesName.add(device.getName() + "(" + device.getSn() + ")");
                            topics.add(device.getDeveloperId() + "/" + device.getSn() + "/" + res.getParameter());
                            NodeProperty prop = AppWidgetHelper.getNodePropertyByDeviceParameter(res.getParameter(), device);
                            String unit = "";
                            if (prop != null) {
                                unit = prop.getUnit();
                            }
                            units.add(unit);
                        }
                    }
                }

                break;
            }
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date().getTime());

        for (int i = 0; i < resourcesName.size(); i++) {
            dbHelper.insert(appWidgetId + "", topics.get(i), widgetTitle, resourcesName.get(i),
                    devicesName.get(i), date, units.get(i), values.get(i),
                    color, alpha, false, AppWidgetType.WGT_MONITOR);
        }

        // It is the responsibility of the configuration activity to update the app widget
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this);
        SimpleTextWidget.updateAppWidget(this, appWidgetManager, appWidgetId);

        // Make sure we pass back the original appWidgetId
        Intent resultValue = new Intent();
        resultValue.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, appWidgetId);
        setResult(RESULT_OK, resultValue);
        finish();
    }

    private void setConfigurationVisibility(boolean isVisible) {
        LinearLayout previewLayout = findViewById(R.id.previewLayout);
        RelativeLayout analyticsLayout = findViewById(R.id.analytics_layout);
        if (isVisible) {
            previewLayout.setVisibility(View.VISIBLE);
            analyticsLayout.setVisibility(View.GONE);
        } else {
            previewLayout.setVisibility(View.GONE);
            analyticsLayout.setVisibility(View.VISIBLE);
        }
    }

    private final AdapterView.OnItemClickListener onItemClicked = new AdapterView.OnItemClickListener() {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            selectedAnalytic = analytics.get(position);
            previewTitle.setText(selectedAnalytic.getTitle());
            setConfigurationVisibility(true);
        }
    };

    public void onRadioButtonClicked(View view) {
        boolean checked = ((RadioButton) view).isChecked();
        // Check which radio button was clicked
        int id = view.getId();
        if (id == R.id.radioButtonDark) {
            if (checked) {
                color = "dark";
                widgetLayout.setBackgroundColor(ContextCompat.getColor(this, R.color.colorContentDark));
            }
        } else if (id == R.id.radioButtonLight) {
            if (checked) {
                color = "light";
                widgetLayout.setBackgroundColor(ContextCompat.getColor(this, R.color.colorContentLight));
            }
        }

        widgetLayout.getBackground().setAlpha(alpha);
    }

    private void handleParsedCompleted(Message msg) {
        if (msg.what == IntentKey.PARSED_COMPLETED) {
            layout.setVisibility(View.VISIBLE);
            progressBar.setVisibility(View.GONE);
            dashboardsAdapter.notifyDataSetChanged();
        }
    }

    private void requestToServer() {
        final OneTimeWorkRequest getDevicesRequest = new OneTimeWorkRequest.Builder(GetDevicesWorker.class)
                .build();
        final OneTimeWorkRequest getAnalyticsRequest = new OneTimeWorkRequest.Builder(GetAnalyticsWorker.class)
                .build();

        final Data data = new Data.Builder()
                .putString(IntentKey.KEY_BROADCAST, IntentKey.APPWIDGET_WORKER_COMPLETED)
                .build();
        final OneTimeWorkRequest getAnalyticsResourcesRequest = new OneTimeWorkRequest.Builder(GetAnalyticsResourcesWorker.class)
                .setInputData(data)
                .build();

        WorkManager.getInstance(this)
                .beginUniqueWork("requestDataForAppWidget", ExistingWorkPolicy.REPLACE, getDevicesRequest)
                .then(getAnalyticsRequest)
                .then(getAnalyticsResourcesRequest)
                .enqueue();
    }

    private final BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equals(IntentKey.APPWIDGET_WORKER_COMPLETED)) {
                String deviceUuid = intent.getExtras().getString(GetDevicesWorker.KEY);
                String analyticUuid = intent.getExtras().getString(GetAnalyticsWorker.KEY);
                String resourceUuid = intent.getExtras().getString(GetAnalyticsResourcesWorker.KEY);

                parseJsonResult(deviceUuid, analyticUuid, resourceUuid);
            }
        }
    };

    private void parseJsonResult(final String deviceUuid, final String analyticUuid, final String resourceUuid) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                // parse devices
                File path = getApplicationContext().getFilesDir();
                String devicesJson = FileUtil.readFromFile(new File(path, deviceUuid + ".json"));

                DeviceResult deviceResult = gson.fromJson(devicesJson, DeviceResult.class);

                if (deviceResult != null && deviceResult.getBody() != null) {
                    devices.clear();
                    devices.addAll(deviceResult.getBody());
                }


                // parse analytic
                String analyticsJson = FileUtil.readFromFile(new File(path, analyticUuid + ".json"));
                AnalyticResult analyticResult = gson.fromJson(analyticsJson, AnalyticResult.class);
                if (analyticResult != null && analyticResult.getBody() != null) {
                    analytics.clear();
                    analytics.addAll(analyticResult.getBody());
                    Collections.sort(analytics);
                }

                // parse resources
                String resourcesJson = FileUtil.readFromFile(new File(path, resourceUuid + ".json"));
                AnalyticsResourcesResult analyticsResourcesResult = gson.fromJson(resourcesJson, AnalyticsResourcesResult.class);
                if (analyticsResourcesResult != null && analyticsResourcesResult.getBody() != null) {
                    analyticResources.clear();
                    analyticResources.addAll(analyticsResourcesResult.getBody());
                }

                // set resource device
                for (AnalyticResource ares : analyticResources) {
                    if (ares.getResources() != null && ares.getResources().size() > 0) {
                        for (Resource res : ares.getResources()) {
                            for (Device dev : devices) {
                                if (res.getDeviceId().equals(dev.getId())) {
                                    res.setDevice(dev);
                                    break;
                                }
                            }

                        }
                    }
                }


                // delete file
                File dir = getFilesDir();
                File fileDevices = new File(dir, deviceUuid + ".json");
                fileDevices.delete();

                File fileAnalytics = new File(dir, analyticUuid + ".json");
                fileAnalytics.delete();

                File fileResources = new File(dir, resourceUuid + ".json");
                fileResources.delete();


                Message msg = new Message();
                msg.what = IntentKey.PARSED_COMPLETED;
                handlerThread.sendMessage(msg);

            }
        }).start();
    }

    @Override
    protected void onDestroy() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(receiver);
        dbHelper.close();
        super.onDestroy();
    }
}
